<?php

namespace Drupal\libraries_delay_load\Form;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form that configures forms module settings.
 */
class JsDelayStrategyForm extends EntityForm {

  /**
   * The entity storage factory for querying.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $entityStorage;

  /**
   * Construct the JsDelayStrategyForm.
   *
   * @param \Drupal\Core\Entity\EntityStorageInterface $entity_storage
   *   An entity query factory for the entity type.
   */
  public function __construct(EntityStorageInterface $entity_storage) {
    $this->entityStorage = $entity_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $form = new static($container->get('entity_type.manager')->getStorage('js_delay_strategy'));
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form = parent::buildForm($form, $form_state);

    $config = $this->entity;

    // Build the form.
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $config->label(),
      '#required' => TRUE,
    ];
    $form['id'] = [
      '#type' => 'machine_name',
      '#title' => $this->t('Machine name'),
      '#default_value' => $config->id(),
      '#machine_name' => [
        'exists' => [$this, 'exists'],
        'replace_pattern' => '([^a-z0-9_]+)',
        'error' => 'The machine-readable name must be unique, and can only contain lowercase letters, numbers, and underscores.',
      ],
      '#disabled' => !$config->isNew(),
    ];

    $form['strategies_title'] = [
      '#type' => 'markup',
      '#markup' => '<h1>Strategy</h1>',
    ];

    $form['excluded'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Select javascript to exclude'),
      '#default_value' => $config->get('excluded'),
      '#description' => $this->t('1 javascript path per line. eg: @example', ['@example' => 'core/assets/vendor/html5shiv/html5shiv.min.js']),
    ];

    $groups = $config->get('groups');

    if (!isset($groups) || count($groups) === 0) {
      // At least 1 group.
      $configNbGroups = 1;
    }
    else {
      $configNbGroups = count($groups);
    }

    // If we added / removed groups through ajax, we should use those.
    $form_state->set('nbGroups', $form_state->get('nbGroups') ?? $configNbGroups);

    $form['groups'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Javascript to delay groups'),
      '#tree' => TRUE,
      '#attributes' => ['id' => 'groups_container'],
    ];

    for ($i = 0; $i < $form_state->get('nbGroups'); $i++) {
      $form['groups'][$i] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Group @number', ['@number' => $i + 1]),
        '#tree' => TRUE,
      ];

      if ($i == 0) {
        $title = $this->t('How long should this group start to be loaded after the window load event is fired (in ms)');
      }
      else {
        $title = $this->t('How long should this group start to be loaded after the load of the previous group is completed (in ms)');
        $description = $this->t('If the previous group is loaded asynchronously, scripts from this group could start to be downloaded before previous ones are executed / finished');
      }
      $form['groups'][$i]['timing'] = [
        '#type' => 'number',
        '#title' => $title,
        '#default_value' => $groups[$i]['timing'] ?? 0,
        '#description' => $description ?? '',
      ];
      $form['groups'][$i]['synchro'] = [
        '#type' => 'radios',
        '#title' => $this->t('Scripts in that group to be loaded one after the other (synchro) or at any time (asynchro)'),
        '#default_value' => $groups[$i]['synchro'] ?? 1,
        '#options' => [
          0 => $this->t('Asynchro'),
          1 => $this->t('Synchro'),
        ],
        '#description' => $this->t('If choosing synchro, scripts will be fetched and executed in the order defined for the group. Then all behaviors will be triggered. If choosing asynchro, scripts will be fetched in parallel and no behaviors will be triggered.'),
      ];
      $form['groups'][$i]['aggregate'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Aggregate all internal files in this group into one?'),
        '#default_value' => $groups[$i]['aggregate'] ?? NULL,
        '#description' => $this->t('Check this box if files need to be loaded together in order to have their behavor work properly. Within a group, aggregated files will be loaded after external ones.'),
      ];

      $form['groups'][$i]['javascript'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Select javascript to delay, 1 javascript path per line'),
        '#default_value' => $groups[$i]['javascript'] ?? NULL,
        '#description' => $this->t('1 javascript path per line. eg: @example', ['@example' => 'core/assets/vendor/html5shiv/html5shiv.min.js']),
      ];
    }

    // Provide an "Add a new group" button.
    $form['groups']['add_group'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add a new group'),
      '#submit' => [[$this, 'addGroup']],
      '#limit_validation_errors' => [],
      '#ajax'   => [
        'callback' => [$this, 'groupsCallback'],
        'wrapper'  => 'groups_container',
      ],
    ];

    if ($form_state->get('nbGroups') > 0) {
      // Provide a "Remove" button for latest selected group.
      $form['groups']['remove_group'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove the last group'),
        '#submit' => [[$this, 'removeGroup']],
        '#limit_validation_errors' => [],
        '#ajax'   => [
          'callback' => [$this, 'groupsCallback'],
          'wrapper'  => 'groups_container',
        ],
      ];
    }

    return $form;
  }

  /**
   * Checks for an existing LibrariesDelayForm config.
   *
   * @param string|int $entity_id
   *   The entity ID.
   * @param array $element
   *   The form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return bool
   *   TRUE if this format already exists, FALSE otherwise.
   */
  public function exists($entity_id, array $element, FormStateInterface $form_state) {
    // Use the query factory to build a new entity query.
    $query = $this->entityStorage->getQuery();

    // Query the entity ID to see if its in use.
    $result = $query->condition('id', $element['#field_prefix'] . $entity_id)
      ->execute();

    // We don't need to return the ID, only if it exists or not.
    return (bool) $result;
  }

  /**
   * Implements callback for Ajax event on add/remove group.
   *
   * @param array $form
   *   From render array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Current state of form.
   *
   * @return array
   *   Groups section of the form.
   */
  public function groupsCallback(array &$form, FormStateInterface $form_state) {
    return $form['groups'];
  }

  /**
   * Adding a new group.
   */
  public function addGroup(array &$form, FormStateInterface $form_state) {
    $nbGroups = $form_state->get('nbGroups') + 1;
    $form_state->set('nbGroups', $nbGroups);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Removing the last group.
   */
  public function removeGroup(array &$form, FormStateInterface $form_state) {
    $nbGroups = $form_state->get('nbGroups') - 1;
    $form_state->set('nbGroups', $nbGroups);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   An associative array containing the current state of the form.
   */
  public function save(array $form, FormStateInterface $form_state) {
    // EntityForm provides us with the entity we're working on.
    $config = $this->getEntity();
    $status = $config->save();

    if ($status == SAVED_UPDATED) {
      $this->messenger()->addMessage($this->t('Strategy %label has been updated.', ['%label' => $config->label()]));
    }
    else {
      $this->messenger()->addMessage($this->t('Strategy %label has been added.', ['%label' => $config->label()]));
    }

    // Clear Caches.
    Cache::invalidateTags(['library_info']);

    // Redirect the user back to the listing route after the save operation.
    $form_state->setRedirect('entity.js_delay_strategy.collection');
  }

}
